@@ -5,7 +5,7 @@
 //
 
 /*
-Copyright or © or Copr. CNRS
+Copyright or © or Copr. Bio++ Development Team
 
 This software is a computer program whose purpose is to estimate
 phylogenies and evolutionary parameters from a dataset according to
@@ -60,6 +60,7 @@ using namespace std;
 #include <Bpp/Seq/Alphabet/Alphabet.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
 #include <Bpp/Seq/Container/SequenceContainerTools.h>
+#include <Bpp/Seq/Container/SiteContainerTools.h>
 #include <Bpp/Seq/App/SequenceApplicationTools.h>
 
 // From PhylLib:
@@ -71,6 +72,7 @@ using namespace std;
 #include <Bpp/Phyl/Model/MarkovModulatedSubstitutionModel.h>
 #include <Bpp/Phyl/Model/SubstitutionModelSet.h>
 #include <Bpp/Phyl/Model/SubstitutionModelSetTools.h>
+#include <Bpp/Phyl/Model/RateDistribution/ConstantRateDistribution.h>
 #include <Bpp/Phyl/Io/Newick.h>
 
 using namespace bpp;
@@ -138,10 +140,12 @@ int main(int args, char ** argv)
     ApplicationTools::displayResult("Writing tagged tree to", treeWIdPath);
     treeWriter.write(ttree, treeWIdPath);
     delete tree;
-    cout << "BppML's done." << endl;
+    cout << "BppAncestor's done." << endl;
     exit(0);
   }
 
+  bool checkTree = ApplicationTools::getBooleanParameter("input.tree.check_root", bppancestor.getParams(), true, "", true, false);
+
   DRTreeLikelihood *tl;
   string nhOpt = ApplicationTools::getStringParameter("nonhomogeneous", bppancestor.getParams(), "no", "", true, false);
   ApplicationTools::displayResult("Heterogeneous model", nhOpt);
@@ -149,22 +153,26 @@ int main(int args, char ** argv)
   SubstitutionModel    *model    = 0;
   SubstitutionModelSet *modelSet = 0;
   DiscreteDistribution *rDist    = 0;
-  unsigned int nbStates;
+  size_t nbStates;
 
   if (nhOpt == "no")
   {  
     model = PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, sites, bppancestor.getParams());
     if (model->getName() != "RE08") SiteContainerTools::changeGapsToUnknownCharacters(*sites);
-    if(model->getNumberOfStates() > model->getAlphabet()->getSize())
+    if (model->getNumberOfStates() > model->getAlphabet()->getSize())
     {
       //Markov-modulated Markov model!
-      rDist = new ConstantDistribution(1., true);
+      rDist = new ConstantRateDistribution();
     }
     else
     {
       rDist = PhylogeneticsApplicationTools::getRateDistribution(bppancestor.getParams());
     }
-    tl = new DRHomogeneousTreeLikelihood(*tree, *sites, model, rDist, true);
+    if (dynamic_cast<MixedSubstitutionModel*>(model))
+      tl = new DRHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, checkTree, true, true);
+    else
+      tl = new DRHomogeneousTreeLikelihood(*tree, *sites, model, rDist, checkTree);
+
     nbStates = model->getNumberOfStates();
   }
   else if (nhOpt == "one_per_branch")
@@ -174,7 +182,7 @@ int main(int args, char ** argv)
     if (model->getNumberOfStates() > model->getAlphabet()->getSize())
     {
       //Markov-modulated Markov model!
-      rDist = new ConstantDistribution(1., true);
+      rDist = new ConstantRateDistribution();
     }
     else
     {
@@ -192,6 +200,8 @@ int main(int args, char ** argv)
     vector<string> globalParameters = ApplicationTools::getVectorParameter<string>("nonhomogeneous_one_per_branch.shared_parameters", bppancestor.getParams(), ',', "");
     modelSet = SubstitutionModelSetTools::createNonHomogeneousModelSet(model, rootFreqs, tree, globalParameters); 
     model = 0;
+    if (dynamic_cast<MixedSubstitutionModelSet*>(modelSet))
+      throw Exception("Non-homogeneous mixed substitution ancestor reconstruction not implemented, sorry!");
     tl = new DRNonHomogeneousTreeLikelihood(*tree, *sites, modelSet, rDist, true);
     nbStates = modelSet->getNumberOfStates();
   }
@@ -208,6 +218,8 @@ int main(int args, char ** argv)
     {
       rDist = PhylogeneticsApplicationTools::getRateDistribution(bppancestor.getParams());
     }
+    if (dynamic_cast<MixedSubstitutionModelSet*>(modelSet))
+      throw Exception("Non-homogeneous mixed substitution ancestor reconstruction not implemented, sorry!");
     tl = new DRNonHomogeneousTreeLikelihood(*tree, *sites, modelSet, rDist, true);
     nbStates = modelSet->getNumberOfStates();
   }
@@ -296,10 +308,10 @@ int main(int args, char ** argv)
       ofstream out(outputFile.c_str(), ios::out);
       TreeTemplate<Node> ttree(*tree);
       vector<Node *> nodes = ttree.getInnerNodes();
-      unsigned int nbNodes = nodes.size();
+      size_t nbNodes = nodes.size();
     
       // Get the rate class with maximum posterior probability:
-      vector<unsigned int> classes = tl->getRateClassWithMaxPostProbOfEachSite();
+      vector<size_t> classes = tl->getRateClassWithMaxPostProbOfEachSite();
       // Get the posterior rate, i.e. rate averaged over all posterior probabilities:
       Vdouble rates = tl->getPosteriorRateOfEachSite();
       // Get the ancestral sequences:
@@ -313,7 +325,7 @@ int main(int args, char ** argv)
       colNames.push_back("lnL");
       colNames.push_back("rc");
       colNames.push_back("pr");
-      for (unsigned int i = 0; i < nbNodes; i++) {
+      for (size_t i = 0; i < nbNodes; i++) {
         Node *node = nodes[i];
         colNames.push_back("max." + TextTools::toString(node->getId()));
         if (probs) {
@@ -339,7 +351,7 @@ int main(int args, char ** argv)
       vector<string> row(colNames.size());
       DataTable* infos = new DataTable(colNames);
     
-      for (unsigned int i = 0; i < sites->getNumberOfSites(); i++)
+      for (size_t i = 0; i < sites->getNumberOfSites(); i++)
       {
         double lnL = tl->getLogLikelihoodForASite(i);
         const Site* currentSite = &sites->getSite(i);
