@@ -5,7 +5,7 @@
 //
 
 /*
-   Copyright or © or Copr. CNRS
+   Copyright or © or Copr. Bio++ Development Team
 
    This software is a computer program whose purpose is to estimate
    phylogenies and evolutionary parameters from a dataset according to
@@ -60,6 +60,7 @@ using namespace std;
 // From SeqLib:
 #include <Bpp/Seq/Alphabet/Alphabet.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
+#include <Bpp/Seq/Container/SiteContainerTools.h>
 #include <Bpp/Seq/SiteTools.h>
 #include <Bpp/Seq/App/SequenceApplicationTools.h>
 
@@ -70,6 +71,9 @@ using namespace std;
 #include <Bpp/Phyl/App/PhylogeneticsApplicationTools.h>
 #include <Bpp/Phyl/OptimizationTools.h>
 #include <Bpp/Phyl/Model.all>
+#include <Bpp/Phyl/Model/Protein/CoalaCore.h>
+#include <Bpp/Phyl/Model/RateDistribution/ConstantRateDistribution.h>
+#include <Bpp/Phyl/Model/FrequenciesSet/MvaFrequenciesSet.h>
 #include <Bpp/Phyl/Io/Newick.h>
 
 using namespace bpp;
@@ -89,11 +93,12 @@ void help()
 int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*       Bio++ Maximum Likelihood Computation, version 1.5.0      *" << endl;
+  cout << "*       Bio++ Maximum Likelihood Computation, version 1.6.0      *" << endl;
   cout << "*                                                                *" << endl;
-  cout << "* Authors: J. Dutheil                       Last Modif. 07/02/11 *" << endl;
+  cout << "* Authors: J. Dutheil                       Last Modif. 29/01/13 *" << endl;
   cout << "*          B. Boussau                                            *" << endl;
-  cout << "*          L. Gueguen                                            *" << endl;
+  cout << "*          L. Guéguen                                            *" << endl;
+  cout << "*          M. Groussin                                           *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
 
@@ -156,7 +161,10 @@ int main(int args, char** argv)
     KeyvalTools::parseProcedure(initBrLenMethod, cmdName, cmdArgs);
     if (cmdName == "Input")
     {
-      // Do nothing!
+      // Is the root has to be moved to the midpoint position along the branch that contains it ? If no, do nothing!
+      string midPointRootBrLengths = ApplicationTools::getStringParameter("midPointRootBrLengths", cmdArgs, "no", "", true, false);
+      if(midPointRootBrLengths == "yes")
+        TreeTools::constrainedMidPointRooting(*tree);
     }
     else if (cmdName == "Equal")
     {
@@ -199,7 +207,7 @@ int main(int args, char** argv)
     {
       TreeTemplate<Node> ttree(*tree);
       vector<Node*> nodes = ttree.getNodes();
-      for (unsigned int i = 0; i < nodes.size(); i++)
+      for (size_t i = 0; i < nodes.size(); i++)
       {
         if (nodes[i]->isLeaf())
           nodes[i]->setName(TextTools::toString(nodes[i]->getId()) + "_" + nodes[i]->getName());
@@ -219,6 +227,7 @@ int main(int args, char** argv)
     string nhOpt = ApplicationTools::getStringParameter("nonhomogeneous", bppml.getParams(), "no", "", true, false);
     ApplicationTools::displayResult("Heterogeneous model", nhOpt);
 
+    bool checkTree    = ApplicationTools::getBooleanParameter("input.tree.check_root", bppml.getParams(), true, "", true, false);
     bool optimizeTopo = ApplicationTools::getBooleanParameter("optimization.topology", bppml.getParams(), false, "", true, false);
     unsigned int nbBS = ApplicationTools::getParameter<unsigned int>("bootstrap.number", bppml.getParams(), 0, "", true, false);
 
@@ -235,14 +244,14 @@ int main(int args, char** argv)
       if (model->getNumberOfStates() >= 2 * model->getAlphabet()->getSize())
       {
         // Markov-modulated Markov model!
-        rDist = new ConstantDistribution(1., true);
+        rDist = new ConstantRateDistribution();
       }
       else
       {
         rDist = PhylogeneticsApplicationTools::getRateDistribution(bppml.getParams());
       }
       if (dynamic_cast<MixedSubstitutionModel*>(model) == 0)
-        tl = new NNIHomogeneousTreeLikelihood(*tree, *sites, model, rDist, true, true);
+        tl = new NNIHomogeneousTreeLikelihood(*tree, *sites, model, rDist, checkTree, true);
       else
         throw Exception("Topology estimation with Mixed model not supported yet, sorry :(");
     }
@@ -253,7 +262,7 @@ int main(int args, char** argv)
       if (model->getNumberOfStates() >= 2 * model->getAlphabet()->getSize())
       {
         // Markov-modulated Markov model!
-        rDist = new ConstantDistribution(1., true);
+        rDist = new ConstantRateDistribution();
       }
       else
       {
@@ -266,25 +275,25 @@ int main(int args, char** argv)
         string compression = ApplicationTools::getStringParameter("likelihood.recursion_simple.compression", bppml.getParams(), "recursive", "", true, false);
         ApplicationTools::displayResult("Likelihood data compression", compression);
         if (compression == "simple")
-          if (dynamic_cast<MixedSubstitutionModel*>(model) == 0)
-            tl = new RHomogeneousTreeLikelihood(*tree, *sites, model, rDist, false, true, false);
+          if (dynamic_cast<MixedSubstitutionModel*>(model))
+            tl = new RHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, checkTree, true, false);
           else
-            tl = new RHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, false, true, false);
+            tl = new RHomogeneousTreeLikelihood(*tree, *sites, model, rDist, checkTree, true, false);
 
         else if (compression == "recursive")
           if (dynamic_cast<MixedSubstitutionModel*>(model) == 0)
-            tl = new RHomogeneousTreeLikelihood(*tree, *sites, model, rDist, false, true, true);
+            tl = new RHomogeneousTreeLikelihood(*tree, *sites, model, rDist, checkTree, true, true);
           else
-            tl = new RHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, false, true, true);
+            tl = new RHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, checkTree, true, true);
 
         else throw Exception("Unknown likelihood data compression method: " + compression);
       }
       else if (recursion == "double")
       {
-        if (dynamic_cast<MixedSubstitutionModel*>(model) == 0)
-          tl = new DRHomogeneousTreeLikelihood(*tree, *sites, model, rDist, true);
+        if (dynamic_cast<MixedSubstitutionModel*>(model))
+          tl = new DRHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, checkTree);
         else
-          tl = new DRHomogeneousMixedTreeLikelihood(*tree, *sites, model, rDist, true);
+          tl = new DRHomogeneousTreeLikelihood(*tree, *sites, model, rDist, checkTree);
       }
       else throw Exception("Unknown recursion option: " + recursion);
     }
@@ -295,7 +304,7 @@ int main(int args, char** argv)
       if (model->getNumberOfStates() >= 2 * model->getAlphabet()->getSize())
       {
         // Markov-modulated Markov model!
-        rDist = new ConstantDistribution(1., true);
+        rDist = new ConstantRateDistribution();
       }
       else
       {
@@ -316,6 +325,17 @@ int main(int args, char** argv)
       {
         rootFreqs = PhylogeneticsApplicationTools::getRootFrequenciesSet(alphabet, sites, bppml.getParams(), rateFreqs);
         stationarity = !rootFreqs;
+        string freqDescription = ApplicationTools::getStringParameter("nonhomogeneous.root_freq", bppml.getParams(), "");
+        if (freqDescription == "MVAprotein")
+        {
+          if (dynamic_cast<CoalaCore*>(model))
+          {
+            dynamic_cast<MvaFrequenciesSet*>(rootFreqs)->setModelName("MVAprotein");
+            dynamic_cast<MvaFrequenciesSet*>(rootFreqs)->initSet(dynamic_cast<CoalaCore*>(model)); 
+          }
+          else
+            throw Exception("The MVAprotein frequencies set at the root can only be used if a COaLA model is used on branches.");
+        }
       }
       ApplicationTools::displayBooleanResult("Stationarity assumed", stationarity);
    
@@ -336,7 +356,7 @@ int main(int args, char** argv)
       }
       else if (recursion == "double")
       {
-        if (dynamic_cast<MixedSubstitutionModelSet*>(modelSet)!=NULL)
+        if (dynamic_cast<MixedSubstitutionModelSet*>(modelSet))
           throw Exception("Double recursion with non homogeneous mixed models is not implemented yet.");
             //            tl = new DRNonHomogeneousMixedTreeLikelihood(*tree, *sites, modelSet, rDist, true);
         else
@@ -351,7 +371,7 @@ int main(int args, char** argv)
       if (modelSet->getNumberOfStates() >= 2 * modelSet->getAlphabet()->getSize())
       {
         // Markov-modulated Markov model!
-        rDist = new ConstantDistribution(1., true);
+        rDist = new ConstantRateDistribution();
       }
       else
       {
@@ -415,15 +435,15 @@ int main(int args, char** argv)
     if (isinf(logL))
     {
       ApplicationTools::displayError("!!! Unexpected initial likelihood == 0.");
-      CodonAlphabet *pca=dynamic_cast<CodonAlphabet*>(alphabet);
+      CodonAlphabet *pca = dynamic_cast<CodonAlphabet*>(alphabet);
       if (pca) {
         bool f = false;
-        unsigned int  s;
-        for (unsigned int i = 0; i < sites->getNumberOfSites(); i++) {
+        size_t s;
+        for (size_t i = 0; i < sites->getNumberOfSites(); i++) {
           if (isinf(tl->getLogLikelihoodForASite(i))) {
             const Site& site=sites->getSite(i);
             s = site.size();
-            for (unsigned int j = 0; j < s; j++) {
+            for (size_t j = 0; j < s; j++) {
               if (pca->isStop(site.getValue(j))) {
                 (*ApplicationTools::error << "Stop Codon at site " << site.getPosition() << " in sequence " << sites->getSequence(j).getName()).endLine();
                 f = true;
@@ -445,7 +465,7 @@ int main(int args, char** argv)
         exit(1);
       } else {
         ApplicationTools::displayBooleanResult("Saturated site removal enabled", true);
-        for (unsigned int i = sites->getNumberOfSites(); i > 0; --i) {
+        for (size_t i = sites->getNumberOfSites(); i > 0; --i) {
           if (isinf(tl->getLogLikelihoodForASite(i - 1))) {
             ApplicationTools::displayResult("Ignore saturated site", sites->getSite(i - 1).getPosition());
             sites->deleteSite(i - 1);
@@ -490,7 +510,7 @@ int main(int args, char** argv)
     ApplicationTools::displayResult("Output estimates to file", parametersFile);
     if (parametersFile != "none")
     {
-      StlOutputStream out(auto_ptr<ostream>(new ofstream(parametersFile.c_str(), ios::out)));
+      StlOutputStream out(new ofstream(parametersFile.c_str(), ios::out));
       out << "# Log likelihood = ";
       out.setPrecision(20) << (-tl->getValue());
       out.endLine();
@@ -531,7 +551,8 @@ int main(int args, char** argv)
       ofstream out(infosFile.c_str(), ios::out);
 
       // Get the rate class with maximum posterior probability:
-      vector<unsigned int> classes = tl->getRateClassWithMaxPostProbOfEachSite();
+      vector<size_t> classes = tl->getRateClassWithMaxPostProbOfEachSite();
+
       // Get the posterior rate, i.e. rate averaged over all posterior probabilities:
       Vdouble rates = tl->getPosteriorRateOfEachSite();
 
