@@ -52,6 +52,7 @@ using namespace std;
 // From SeqLib:
 #include <Bpp/Seq/SiteTools.h>
 #include <Bpp/Seq/Alphabet/Alphabet.h>
+#include <Bpp/Seq/Alphabet/AlphabetTools.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
 #include <Bpp/Seq/App/SequenceApplicationTools.h>
 #include <Bpp/Seq/Io.all>
@@ -313,7 +314,7 @@ int main(int args, char** argv)
         throw Exception("'RemoveColumnsWithStop' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
       }
 
-      for (unsigned int i = sites->getNumberOfSites(); i > 0; i--)
+      for (size_t i = sites->getNumberOfSites(); i > 0; i--)
       {
         if (SiteTools::hasStopCodon(sites->getSite(i-1)))
           sites->deleteSite(i - 1);
@@ -331,10 +332,10 @@ int main(int args, char** argv)
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         BasicSequence seq = sequences->getSequence(i);
-        unsigned int len = seq.size();
+        size_t len = seq.size();
         SequenceTools::getCDS(seq, false, true, true, false);
         if (aligned) {
-          for (unsigned int c = seq.size(); c < len; ++c)
+          for (size_t c = seq.size(); c < len; ++c)
             seq.addElement(seq.getAlphabet()->getGapCharacterCode());
         }
         sc->addSequence(seq, false);
@@ -390,7 +391,7 @@ int main(int args, char** argv)
       if (maxGapOption[maxGapOption.size()-1] == '%')
       {
         double gapFreq = TextTools::toDouble(maxGapOption.substr(0, maxGapOption.size()-1)) / 100.;
-        for (unsigned int i = sites->getNumberOfSites(); i > 0; i--)
+        for (size_t i = sites->getNumberOfSites(); i > 0; i--)
         {
           map<int, double> freqs;
           SiteTools::getFrequencies(sites->getSite(i - 1), freqs);
@@ -399,10 +400,10 @@ int main(int args, char** argv)
       }
       else
       {
-        unsigned int gapNum=TextTools::to<unsigned int>(maxGapOption);
-        for (unsigned int i = sites->getNumberOfSites(); i > 0; i--)
+        size_t gapNum = TextTools::to<size_t>(maxGapOption);
+        for (size_t i = sites->getNumberOfSites(); i > 0; i--)
         {
-          map<int, unsigned int> counts;
+          map<int, size_t> counts;
           SiteTools::getCounts(sites->getSite(i - 1), counts);
           counts[-1]; //Needed in case this entry does not exist in the map. This will set it to 0.
           if (counts[-1] > gapNum) sites->deleteSite(i-1);
